﻿using System;
using System.Runtime.InteropServices;

namespace ConsoleApplication1
{
    class Program
    {
        [StructLayout(LayoutKind.Sequential)]
        public struct LOCALGROUP_INFO_0
        {
            public IntPtr lgrpi0_name;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct LOCALGROUP_INFO_1
        {
            public IntPtr lgrpi1_name;
            public IntPtr lgrpi1_comment;
        }

        public const int ERROR_INVALID_PARAMETER = 87;
        public const int ERROR_NO_SUCH_ALIAS = 1376;
        public const int NERR_InvalidComputer = 2351;
        public const int ERROR_ACCESS_DENIED = 5;
        public const int NERR_Success = 0;
        public const int NERR_NotPrimary = 2226;
        public const int NERR_GroupNotFound = 2220;

        [DllImport("Netapi32.dll", EntryPoint = "NetLocalGroupSetInfo", CharSet = CharSet.Unicode, SetLastError = true)]
        extern static int NetLocalGroupSetInfo(
            string serverName, string groupName, int level,
            IntPtr buf, ref int param_error);

        static void Main(string[] args)
        {
            string oldGroupName = "MyGroup";
            
            string newGroupName = "TestGroup";
            string newGroupComment = "Test Group";

            int result = ChangeGroupName(oldGroupName, newGroupName);
            Console.WriteLine("ChangeGroupName: " + result);

            ChangeGroupComment(newGroupName, newGroupComment);
            Console.WriteLine("ChangeGroupComment: " + result);
        }

        private static int ChangeGroupComment(string groupName, string newGroupComment)
        {
            LOCALGROUP_INFO_1 info = new LOCALGROUP_INFO_1();
            info.lgrpi1_name = Marshal.StringToHGlobalUni(groupName);
            info.lgrpi1_comment = Marshal.StringToHGlobalUni(newGroupComment);

            int size = Marshal.SizeOf(info);
            IntPtr pData = Marshal.AllocHGlobal(size);

            Marshal.StructureToPtr(info, pData, false);

            int param_err = 0;
            int result = NetLocalGroupSetInfo(null, groupName, 1, pData, ref param_err);

            Marshal.FreeHGlobal(pData);
            Marshal.FreeHGlobal(info.lgrpi1_name);
            Marshal.FreeHGlobal(info.lgrpi1_comment);

            return result;
        }

        private static int ChangeGroupName(string oldGroupName, string newGroupName)
        {
            LOCALGROUP_INFO_0 info = new LOCALGROUP_INFO_0();
            info.lgrpi0_name = Marshal.StringToHGlobalUni(newGroupName);

            int size = Marshal.SizeOf(info);
            IntPtr pData = Marshal.AllocHGlobal(size);

            Marshal.StructureToPtr(info, pData, false);

            int param_err = 0;
            int result = NetLocalGroupSetInfo(null, oldGroupName, 0, pData, ref param_err);

            Marshal.FreeHGlobal(pData);
            Marshal.FreeHGlobal(info.lgrpi0_name);

            return result;
        }
    }
}
